from functools import wraps
import time
import json
import re
import webbrowser
import ctypes
import os
from datetime import datetime
import shutil
from dvatioff_audio.waapi.waapi_utils import get_wwise_originals_sfx_path
from dvatioff_audio.voice.voice_utils import text_distance
import sys
import subprocess


def retry(exceptions, tries=5, delay=1):
    """
    重试装饰器，当函数抛出指定的异常时，自动重试。
    :param exceptions: 触发重试的异常类型
    :param tries: 最大重试次数
    :param delay: 初始延迟时间
    """

    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            mtries, mdelay = tries, delay
            while mtries > 1:
                try:
                    return func(*args, **kwargs)
                except exceptions as e:
                    print(f"警告：{func.__name__} - {e}, 正在重试...")
                    time.sleep(mdelay)
                    mtries -= 1
            return func(*args, **kwargs)

        return wrapper

    return decorator


def load_dict_from_json_file(file_path):
    """
    从 json 文件中加载数据
    """
    with open(file_path, "r", encoding="utf-8") as f:
        data = json.load(f)

    return data


def save_dict_to_json_file(file_path, data):
    """
    将数据保存到 json 文件中
    """
    with open(file_path, "w", encoding="utf-8") as f:
        json.dump(data, f, ensure_ascii=False, indent=4)


def is_random(name):
    """
    是否是随机类声音
    """
    tail = name.split("_")[-1]
    pattern = r"0[1-9]$"
    return bool(re.match(pattern, tail))


def is_loop(name):
    """
    是否是 Loop 类声音
    """
    tail = name.split("_")[-1]
    pattern1 = r"Loop$"
    pattern2 = r"Lp$"
    return bool(re.match(pattern1, tail) or re.match(pattern2, tail))


def is_voice(name):
    """
    是否是 voice 类声音
    """
    head = name.split("_")[0]
    pattern = r"VO$"
    return bool(re.fullmatch(pattern, head))


def is_death(name):
    """
    是否是死亡类声音
    """
    tail = name.split("_")[-1]
    pattern = r"Death$"
    return bool(re.match(pattern, tail))


def open_url(url):
    """
    打开指定的 URL
    """
    webbrowser.open(url)


def minimize_console():
    """
    最小化控制台窗口
    """
    kernel32 = ctypes.WinDLL('kernel32')
    user32 = ctypes.WinDLL('user32')

    SW_MINIMIZE = 6

    console_window = kernel32.GetConsoleWindow()
    if console_window:
        user32.ShowWindow(console_window, SW_MINIMIZE)


def get_wav_path_in_folder(folder_path):
    """
    获取文件夹中的 wav 文件路径
    """
    wav_path = {}
    for root, dirs, files in os.walk(folder_path):
        for file in files:
            if file.endswith(".wav"):
                file_name = os.path.splitext(file)[0]
                wav_path[file_name] = os.path.join(root, file)

    return wav_path


def replace_file(cur_file, new_file):
    """
    替换文件
    """
    if os.path.exists(cur_file):
        os.remove(cur_file)
        print(f"文件 {cur_file} 已删除。")
    else:
        print(f"文件 {cur_file} 不存在。")
        return False

    shutil.copy2(new_file, cur_file)
    print(f"文件 {new_file} 已复制到 {cur_file}。")

    return True


def match_replaced_audios(client, folder_path):
    """
    遍历给定文件夹中的 .wav 文件，搜索 Originals 文件夹中的同名文件，返回匹配和不匹配的文件的路径
    """
    matched_files = []
    unmatched_files = []
    path_originals = get_wwise_originals_sfx_path(client)
    new_wav_path = get_wav_path_in_folder(folder_path)
    ori_wav_path = get_wav_path_in_folder(path_originals)

    for key, value in new_wav_path.items():
        if key in ori_wav_path.keys():
            matched_files.append([value, ori_wav_path[key]])
        else:
            root = os.path.dirname(value)
            unmatched_files.append([root, key])

    if unmatched_files:
        return matched_files, unmatched_files, ori_wav_path
    else:
        return matched_files, None, None


def replace_audios(match_files):
    """
    替换匹配的文件
    """
    try:
        for match_file in match_files:
            cur_file = match_file[1]
            new_file = match_file[0]
            replace_file(cur_file, new_file)
        return True
    except Exception as e:
        print(f"替换文件时出错: {e}")
        return False


def create_placeholder_folder():
    """创建一个临时占位文件夹"""
    folder_path = f'\\placeholder'
    cur_path = os.getcwd()
    abs_path = cur_path + folder_path

    if not os.path.exists(abs_path):
        os.makedirs(abs_path)

    return abs_path


def delete_files_name_with_keyword(folder_path, keyword):
    """
    删除指定文件夹中，文件名包含指定关键字的文件
    """
    for root, dirs, files in os.walk(folder_path):
        for file_name in files:
            if keyword in file_name:
                file_path = os.path.join(root, file_name)
                try:
                    os.remove(file_path)  # 删除文件
                    print(f"已删除文件: {file_path}")
                except OSError as e:
                    print(f"删除文件 {file_path} 失败: {e}")
        for dir_name in dirs:
            if keyword in dir_name:
                dir_path = os.path.join(root, dir_name)
                try:
                    shutil.rmtree(dir_path)  # 删除文件夹及其内容
                    print(f"已删除文件夹: {dir_path}")
                except OSError as e:
                    print(f"删除文件夹 {dir_path} 失败: {e}")


def clear_folder(folder_path):
    """
    清空文件夹
    """
    for root, dirs, files in os.walk(folder_path):
        for file in files:
            os.remove(os.path.join(root, file))
    return True


def get_desktop_path():
    """
    获取桌面路径
    """
    return os.path.join(os.path.expanduser("~"), 'Desktop')


def get_current_time():
    # 使用 datetime.now() 获取当前时间
    return datetime.now().strftime("%Y-%m-%d %H:%M:%S")


def get_most_similar_name(file_name, name_list):
    """
    获取与给定文件名最相似的名称
    """

    min_distance = 100
    most_similar_name = ""

    for name in name_list:
        distance = text_distance(file_name, name)
        if distance < min_distance:
            min_distance = distance
            most_similar_name = name

    return most_similar_name


def keep_only_character(text):
    """
    定义一个正则表达式模式，匹配所有标点符号
    # 包括英文和CJK（中文、日文、韩文）标点符号
    """

    pattern = re.compile(r'[!"#$%&\'()*+,-./:;<=>?@\[\\\]^`{|}~，？！…。、【】；‘’“”·（）— ]')

    # 使用正则表达式的sub方法将这些标点符号替换为空
    text = pattern.sub('', text)

    # 将连续的空白字符替换为单个空格
    text = re.sub(r'\s+', ' ', text)

    # 移除头部和尾部的空白字符
    return text.strip()


def filename_validation(folder_path, name_list):
    """
    验证文件名是否符合规范, 返回不符合规范的文件名
    """

    filename_pair = []
    empty_folder = True
    for root, dirs, files in os.walk(folder_path):
        for file in files:
            if file.endswith('.wav'):
                empty_folder = False
                file_name, file_extension = os.path.splitext(file)

                # 初步修改文件名，去除冗余符号
                cur_file_path = os.path.join(root, file_name + file_extension)
                file_name = keep_only_character(file_name)
                new_file_path = os.path.join(root, file_name + file_extension)
                if cur_file_path != new_file_path:
                    os.rename(cur_file_path, new_file_path)

                # 初步修改后，文件名拼写仍有错误
                if file_name not in name_list:
                    most_similar_name = get_most_similar_name(file_name, name_list)
                    filename_pair.append([file_name, most_similar_name, root, file_extension])

    return filename_pair, empty_folder


def run_exe(relative_path):
    """
    运行 exe 文件
    """
    file_type = relative_path.split('.')[-1]
    absolute_path = os.path.abspath(relative_path)
    cmd = f'"{absolute_path}"' if file_type == 'bat' else f'"{relative_path}"'

    CREATE_NEW_CONSOLE = 0x10
    cur_work_path = relative_path[:relative_path.rfind('\\')]
    subprocess.Popen(cmd, cwd=cur_work_path, creationflags=CREATE_NEW_CONSOLE)


def open_excel(relative_path):
    """
    打开 Excel 文件
    """
    absolute_path = os.path.abspath(relative_path)
    os.startfile(absolute_path)


def open_hyperlink(url):
    """
    打开超链接，根据操作系统的不同，使用不同的方式打开
    """
    if sys.platform == "win32":
        os.startfile(url)
    else:
        opener = "open" if sys.platform == "darwin" else "xdg-open"
        subprocess.call([opener, url])



"""
此脚本中的类用于启动一个子线程，实时获取工具与 Wwise 工程 waapi 通信接口的连接状态，并通过 Signal 信号将连接状态的信息传递给主线程
"""


import time
import asyncio
import waapi
from PySide6.QtCore import Signal, QRunnable, QObject
import threading
from dvatioff_audio.waapi.waapi_utils import get_connected_project_name_info


# 用于发送连接状态的信号
class SignalConnectionStatus(QObject):
    signal = Signal(bool)


# 用于发送 waapi 客户端的信号
class SignalClient(QObject):
    signal = Signal(object)


# 用于发送 Wwise 工程名称的信号
class SignalProjectName(QObject):
    signal = Signal(str, str)


class WaapiConnectWorker(QRunnable):
    signal_connection_status = SignalConnectionStatus()
    signal_waapi_client = SignalClient()
    signal_project_name = SignalProjectName()

    def __init__(self):
        super().__init__()
        self.client = None
        self.running = threading.Event()
        self.running.set()
        self.connect_status = False
        self.connect_changed = False
        self.worker_thread = None

    def run(self):
        self.worker_thread = threading.Thread(target=self.run_in_thread)
        self.worker_thread.start()

    def run_in_thread(self):
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)

        while self.running.is_set():
            self.connect_to_waapi()
            time.sleep(0.5)

        loop.close()

    def connect_to_waapi(self):
        """
        尝试连接到 waapi 并更新UI信号。如果已连接，直接返回
        """
        if self.client and self.client.is_connected():
            return

        self.reset_client()

        try:
            self.client = waapi.WaapiClient()
            if self.client and self.client.is_connected():
                self.handle_successful_connection()
            else:
                self.handle_failed_connection()
        except waapi.CannotConnectToWaapiException as e:
            print(f"无法连接到 waapi: {e}")
            self.handle_failed_connection()

    def reset_client(self):
        """
        重置客户端状态，清除任何现有的客户端引用
        """
        if self.client:
            self.client = None
        self.signal_waapi_client.signal.emit(self.client)
        self.signal_connection_status.signal.emit(False)

    def handle_successful_connection(self):
        """
        处理成功连接的情况，获取项目名称并发送相关信号
        """
        try:
            project_name, wwise_version = get_connected_project_name_info(self.client)
            self.signal_project_name.signal.emit(project_name, wwise_version)
            self.signal_connection_status.signal.emit(True)
            self.signal_waapi_client.signal.emit(self.client)
        except Exception as e:
            print(f"获取项目名称失败: {e}")
            self.reset_client()

    def handle_failed_connection(self):
        """
        处理连接失败的情况，重置客户端并发送失败信号
        """
        self.reset_client()

    def stop(self):
        self.running.clear()
        if self.client:
            self.client.disconnect()
            self.client = None
        if self.worker_thread:
            self.worker_thread.join()
            self.worker_thread = None



from PySide6.QtCore import Qt, Signal
from PySide6.QtGui import QIcon
from PySide6.QtWidgets import QWidget
import dvatioff_audio.gui.gui_css as css
from dvatioff_audio.globals import PATH_ICON_WARNING
from dvatioff_audio.gui.gui_utils import create_label, create_button, add_widgets_to_vhboxlayout, create_layout


class NotificationWindow(QWidget):
    signal_confirmed = Signal()

    def __init__(self, title, message, icon_path=PATH_ICON_WARNING):
        super().__init__()
        self.title = title
        self.message = message
        self.setWindowIcon(QIcon(icon_path))
        self.setWindowModality(Qt.ApplicationModal)
        self.init_UI()
        self.show()

    def init_UI(self):
        self.setWindowTitle(self.title)
        self.setWindowFlags(Qt.WindowStaysOnTopHint)
        self.setFixedSize(400, 200)

        layout = create_layout('vbox', self)

        label_message = create_label(self.message, css.LABEL_STYLE_NORMAL, alignment=Qt.AlignCenter)
        button_ok = create_button('确认', css.BUTTON_STYLE_PINK)
        button_ok.clicked.connect(self.confirm)

        add_widgets_to_vhboxlayout(layout, [
            [label_message],
            [button_ok]
        ])

    def confirm(self):
        self.signal_confirmed.emit()
        self.close()

    def change_icon(self, icon_path):
        self.setWindowIcon(QIcon(icon_path))



import re
from jiwer import wer
from pypinyin import lazy_pinyin
import Levenshtein
import pykakasi
import numpy as np
import wavio
import os
import shutil
import pandas as pd
from openpyxl import load_workbook
from openpyxl.styles import Font
from azure_speech2text import speech_to_text
from dvatioff_audio.utils import get_desktop_path


def generate_sine_wave(frequency=440.0, duration=1, rate=8000):
    """生成一个正弦波形"""
    t = np.linspace(0, duration, int(duration * rate), endpoint=False)
    return np.sin(2 * np.pi * frequency * t)


def save_sine_wave_to_wav(filename, frequency=440.0, duration=1, rate=8000):
    """保存正弦波到.wav文件"""
    sine_wave = generate_sine_wave(frequency, duration, rate)
    wavio.write(filename, sine_wave, rate, sampwidth=3)


def chinese_to_pinyin(text):
    """
    把中文转化为拼音
    """
    lazy_pinyin_text = lazy_pinyin(text)
    sentence = " ".join(lazy_pinyin_text)

    # 将连续的空白字符替换为单个空格
    sentence = re.sub(r'\s+', ' ', sentence)

    return sentence


def japanese_to_kata(text):
    """
    将日文转化为片假名
    """
    kks = pykakasi.kakasi()
    kata_text = kks.convert(text)
    kata = ''

    for item in kata_text:
        kata += item['kana']

    return kata


def japanese_to_romaji(text):
    """
    将日文转化为罗马音
    """
    kks = pykakasi.kakasi()
    kks_text = kks.convert(text)
    romaji = ''

    for item in kks_text:
        romaji += item['hepburn']

    return romaji


def remove_text_within_brackets(text):
    """
    去除文本中括号内的内容，包括括号本身，这里的括号包括半角括号 () 和全角括号 （）和方括号 []
    """
    pattern = re.compile(r'\(.*?\)|（.*?）|\[.*?\]')

    return pattern.sub('', text)


def replace_punctuation_to_space(text):
    """
    去除文本中所有的标点符号
    """
    text = text.replace('\n', '')

    # 正则表达式匹配所有标点符号
    pattern = re.compile(r'[!"#$%&\'()*+,-./:;<=>?@\[\\\]^_`{|}~，？！…。、【】；‘’“”·（）—―～「」]')

    text = pattern.sub(' ', text)
    text = re.sub(r'\s+', ' ', text)

    return text.strip()


def replace_punctuation_to_empty(text):
    """
    去除文本中的标点符号
    """

    # 将'\n' 和 ' ' 替换为''
    text = text.replace('\n', '')
    text = text.replace(' ', '')

    # 正则表达式匹配所有标点符号
    pattern = re.compile(r'[!"#$%&\'()*+,-./:;<=>?@\[\\\]^_`{|}~，？！…。、【】；‘’“”·（）—―「」]')
    text = pattern.sub('', text)

    return text


def remove_all_space(text):
    """
    去除文本中的所有空格
    """
    text = text.replace(' ', '')

    return text


def text_distance(text1, text2):
    """
    计算两个文本之间的 Levenshtein 距离
    """
    distance = Levenshtein.distance(text1, text2)

    return distance


def text_wrong_proportion(text1, text2):
    """
    使用 Jiwer 计算两个字符串之间的词错误率
    """
    proportion = wer(text1, text2)

    return proportion


def text_speech_verify_worker(folder_path, legal_audio_stt_dict, language, update_queue):
    """
    遍历文件夹中的所有 wav 文件，使用 Azure Speech2Text 接口识别音频中的文本，并与合法的文本进行比对
    """

    total_index = 0
    total_file_num = len(legal_audio_stt_dict)

    for root, dirs, files in os.walk(folder_path):
        result = {}
        cur_index = 0
        cur_folder_file_num = 0

        # 统计当前子文件夹中的合法 wav 文件数量
        for file in files:
            if file.endswith('.wav'):
                file_name, file_extension = os.path.splitext(file)
                if file_name in legal_audio_stt_dict.keys():
                    cur_folder_file_num += 1

        for index, file in enumerate(files, start=1):
            if file.endswith('.wav'):
                cur_index += 1
                total_index += 1
                file_name, file_extension = os.path.splitext(file)
                if file_name not in legal_audio_stt_dict.keys():
                    continue
                file_path = os.path.join(root, file)
                # 获取父目录的完整路径
                parent_directory_path = os.path.dirname(file_path)
                # 获取父目录的名称
                parent_directory_name = os.path.basename(parent_directory_path)
                # 识别音频中的文本
                speech_text = speech_to_text(file_path, language)

                result["parent_directory_name"] = parent_directory_name
                result["file_name"] = file_name
                result["file_path"] = file_path
                result["total_index"] = total_index
                result["total_file_num"] = total_file_num
                result["cur_index"] = cur_index
                result["cur_folder_file_num"] = cur_folder_file_num

                if speech_text is None:
                    true_text = "Azure 接口调用失败"
                    result["true_text"] = true_text
                    update_queue.put(result)
                    continue

                # 去除 stt文本/合法文本 中所有的标点符号，如果语言为英语还须将文本转化为小写
                speech_text = replace_punctuation_to_empty(speech_text.lower()) if language == "English" else replace_punctuation_to_empty(speech_text)
                true_text = replace_punctuation_to_empty(
                    legal_audio_stt_dict[file_name]["dialogue"]).lower() if language == "English" else replace_punctuation_to_empty(
                    legal_audio_stt_dict[file_name]["dialogue"])

                result["true_text"] = true_text
                result["speech_text"] = speech_text

                if language == "Chinese":
                    # 将中文文本转化为拼音
                    true_pinyin = chinese_to_pinyin(true_text)
                    speech_pinyin = chinese_to_pinyin(speech_text)

                    # 计算两个文本之间的词错误率
                    wer = text_wrong_proportion(true_pinyin, speech_pinyin)
                    wer = 1 if wer >= 1 else wer

                    result["true_pinyin"] = true_pinyin
                    result["speech_pinyin"] = speech_pinyin
                    result["wer"] = wer
                elif language == "Japanese":
                    # 将日文文本转化为片假名和罗马音
                    true_kata = japanese_to_kata(true_text)
                    file_kata = japanese_to_kata(speech_text)
                    true_kata_romaji = japanese_to_romaji(true_kata)
                    file_kata_romaji = japanese_to_romaji(file_kata)

                    # 计算两个文本之间的词错误率
                    wer_text = text_wrong_proportion(true_text, speech_text)
                    wer_kata = text_wrong_proportion(true_kata, file_kata)
                    distance = text_distance(true_kata_romaji, file_kata_romaji)
                    distance = distance if distance > 0 else 0

                    # 计算两个文本之间的 Levenshtein 距离
                    text_spaceless = true_kata_romaji.replace(' ', '')
                    text_length = len(text_spaceless)

                    # 计算最终的词错误率
                    wer_romaji = distance / text_length if text_length > 0 else 1
                    wer_romaji = wer_romaji if wer_romaji >= 0 else 1
                    wer = min(wer_text, wer_kata, wer_romaji)
                    wer = 1 if wer >= 1 else wer
                    result["true_kata"] = true_kata
                    result["speech_kata"] = file_kata
                    result["true_kata_romaji"] = true_kata_romaji
                    result["speech_kata_romaji"] = file_kata_romaji
                    result["wer"] = wer
                elif language == "Korean":
                    # 去除文本中所有的空格
                    distance = text_distance(remove_all_space(true_text), remove_all_space(speech_text))

                    # 计算两个文本之间的 Levenshtein 距离
                    distance = distance if distance > 0 else 0
                    text_spaceless = remove_all_space(true_text)
                    text_length = len(text_spaceless)

                    # 计算两个文本之间的词错误率
                    wer = distance / text_length if text_length > 0 else 1
                    wer = 1 if wer >= 1 else wer
                    result["wer"] = wer
                elif language == "Russian":
                    # 计算两个文本之间的词错误率
                    distance = text_distance(true_text, speech_text)
                    text_spaceless = true_text.replace(' ', '')
                    text_length = len(text_spaceless)

                    # 计算两个文本之间的词错误率
                    wer = distance / text_length if text_length > 0 else 1
                    wer = 1 if wer >= 1 else wer
                    result["wer"] = wer
                else:
                    # 计算两个文本之间的词错误率
                    wer = text_wrong_proportion(true_text, speech_text)
                    wer = 1 if wer >= 1 else wer
                    result["wer"] = wer

                update_queue.put(result)


def output_text_speech_verify_result(folder_name, file_list, error_log, language):
    """
    整合 Speech2Text 的结果中匹配率异常的 wav 文件，并生成 xlsx 错误日志，发送至桌面
    """

    if language == "Chinese":
        hyperlink_column = 7
    elif language == "Japanese":
        hyperlink_column = 9
    else:
        hyperlink_column = 5

    new_paths = []
    # 获取桌面路径
    desktop_path = get_desktop_path()

    desktop_folder_name = f'{language}-{folder_name}-错误文件'
    desktop_folder_path = os.path.join(desktop_path, desktop_folder_name)

    # 在桌面创建新文件夹
    if not os.path.exists(desktop_folder_path):
        os.makedirs(desktop_folder_path)

    log_name = '错误日志.xlsx'
    log_path = os.path.join(desktop_folder_path, log_name)

    for file in file_list:
        # 仅当文件存在时执行复制操作
        if os.path.isfile(file):
            shutil.copy(file, desktop_folder_path)
            new_paths.append(os.path.basename(file))

    # 将原始数据分割为单独的记录
    records = error_log.strip().split('\n\n')

    # 解析每个记录
    parsed_data = []
    for record in records:
        lines = record.split('\n')
        record_dict = {}
        for line in lines:
            line = line.replace('|', '\n')
            key, value = line.split(': ')
            record_dict[key] = value
        parsed_data.append(record_dict)

    # 创建DataFrame
    df = pd.DataFrame(parsed_data)

    # 导出到Excel
    df.to_excel(log_path, index=False)

    # 使用openpyxl打开Excel文件
    wb = load_workbook(log_path)
    ws = wb.active

    # 为每个文件路径设置超链接
    for row, file_path in enumerate(new_paths, start=2):  # start=1 从第一行开始
        cell = ws.cell(row=row, column=hyperlink_column)  # 第5列
        cell.value = "Open File"
        cell.hyperlink = file_path
        cell.font = Font(color='0000FF', underline='single')

    # 设置列宽
    ws.column_dimensions['A'].width = 30
    ws.column_dimensions['B'].width = 50
    ws.column_dimensions['C'].width = 50
    ws.column_dimensions['D'].width = 30
    ws.column_dimensions['E'].width = 16

    # 保存更改
    wb.save(log_path)



from PySide6.QtCore import Qt, QUrl, QTimer, QObject, QEvent, QPoint
from PySide6.QtGui import QDesktopServices, QIntValidator
from PySide6.QtWidgets import QLabel, QFrame, QPushButton, QLineEdit, QComboBox, QTextEdit, QTextBrowser, QVBoxLayout, QHBoxLayout, QGridLayout, \
    QFormLayout, QCheckBox

class CustomTooltip(QFrame):
    def __init__(self, text, parent=None):
        super().__init__(parent, Qt.ToolTip)
        self.setFrameShape(QFrame.Box)
        self.setFrameShadow(QFrame.Plain)
        self.setLineWidth(1)
        self.setStyleSheet("""
            QFrame {
                background-color: #9998a7;
                color: white;
                border-radius: 2px;
                padding: 1px;
            }
            QLabel {
                font-family: "Microsoft YaHei";
                font-size: 9pt;
            }
        """)
        self.label = QLabel(text)
        layout = QVBoxLayout()
        layout.addWidget(self.label)
        self.setLayout(layout)
        self.adjustSize()


class TooltipManager(QObject):
    def __init__(self, widget, tooltip_text, hover_delay=100):
        super().__init__(widget)
        self.widget = widget
        self.tooltip_text = tooltip_text
        self.hover_delay = hover_delay  # 毫秒

        self.tooltip = CustomTooltip(self.tooltip_text, parent=widget)
        self.tooltip.hide()

        self.hover_timer = QTimer(self)
        self.hover_timer.setSingleShot(True)
        self.hover_timer.timeout.connect(self.show_tooltip)

        self.hide_timer = QTimer(self)
        self.hide_timer.setSingleShot(True)
        self.hide_timer.timeout.connect(self.hide_tooltip)

        self.widget.installEventFilter(self)

    def eventFilter(self, obj, event):
        if obj == self.widget:
            if event.type() == QEvent.Enter:
                self.hover_timer.start(self.hover_delay)
            elif event.type() == QEvent.Leave:
                self.hover_timer.stop()
                self.hide_timer.stop()
                self.hide_tooltip()
        return super().eventFilter(obj, event)

    def show_tooltip(self):
        if not self.tooltip.isVisible():
            # 获取全局位置，并调整 tooltip 位置
            global_pos = self.widget.mapToGlobal(QPoint(0, self.widget.height()))
            self.tooltip.move(global_pos)
            self.tooltip.show()

    def hide_tooltip(self):
        if self.tooltip.isVisible():
            self.tooltip.hide()


def clear_layout(layout):
    """
    清空布局及其中的所有控件
    """
    if layout is not None:
        while layout.count():
            item = layout.takeAt(0)
            widget = item.widget()
            if widget is not None:
                widget.deleteLater()
            else:
                clear_layout(item.layout())
            layout.removeItem(item)


def enable_button(button):
    """
    启用按钮
    """
    button.setDisabled(False)


def enable_all_buttons(buttons):
    """
    启用所有按钮
    """
    for button in buttons:
        button.setDisabled(False)


def disable_button(button):
    """
    禁用按钮
    """
    button.setDisabled(True)


def disable_all_buttons(buttons):
    """
    禁用所有按钮
    """
    for button in buttons:
        button.setDisabled(True)


def create_layout(layout_type, parent=None, spacing=None, margin=None):
    """
    创建布局
    """
    if layout_type == "vbox":
        layout = QVBoxLayout(parent)
    elif layout_type == "hbox":
        layout = QHBoxLayout(parent)
    elif layout_type == "grid":
        layout = QGridLayout(parent)
    elif layout_type == "form":
        layout = QFormLayout(parent)
    else:
        return None
    if parent:
        parent.setLayout(layout)
    if spacing:
        layout.setSpacing(spacing)
    if margin:
        left, top, right, bottom = margin
        layout.setContentsMargins(left, top, right, bottom)

    return layout


def create_label(text, style=None, width=None, height=None, alignment=None, external_link=False, link_activate=False, invisible=False, tooltip=""):
    """
    创建标签
    """
    label = QLabel(text)
    if style:
        label.setStyleSheet(style)
    if width:
        label.setFixedWidth(width)
    if height:
        label.setFixedHeight(height)
    if alignment:
        label.setAlignment(alignment)
    if external_link:
        label.setOpenExternalLinks(True)
    if link_activate:
        label.linkActivated.connect(open_local_url)
    if invisible:
        label.setVisible(False)
    if tooltip:
        TooltipManager(label, tooltip)

    return label


def create_line(height=None):
    """
    创建分割线
    """
    line = QFrame()
    line.setFrameShape(QFrame.HLine)
    line.setFrameShadow(QFrame.Sunken)
    if height:
        line.setFixedHeight(height)

    return line


def create_button(text, style=None, width=None, height=None, enabled=True, clicked=None, invisible=False, tooltip=""):
    """
    创建按钮
    """
    button = QPushButton(text)
    if style:
        button.setStyleSheet(style)
    if width:
        button.setFixedWidth(width)
    if height:
        button.setFixedHeight(height)
    if clicked:
        button.clicked.connect(clicked)
    if invisible:
        button.setVisible(False)
    if tooltip:
        TooltipManager(button, tooltip)
    button.setEnabled(enabled)

    return button


def create_lineEdit(text="", width=None, style=None, text_changed=None, int_validator=False, read_only=False, invisible=False, tooltip=""):
    """
    创建文本框
    """
    lineEdit = QLineEdit(text)
    if read_only:
        lineEdit.setStyleSheet("background-color: #c8c8c8;")
        lineEdit.setReadOnly(True)
    if width:
        lineEdit.setFixedWidth(width)
    if style:
        lineEdit.setStyleSheet(style)
    if text_changed:
        lineEdit.textChanged.connect(text_changed)
    if int_validator:
        lineEdit.setValidator(QIntValidator())
    if invisible:
        lineEdit.setVisible(False)
    if tooltip:
        TooltipManager(lineEdit, tooltip)

    return lineEdit


def create_comboBox(width=None, style=None, items=None, current_changed=None, invisible=False, tooltip=""):
    """
    创建下拉框
    """
    comboBox = QComboBox()
    if width:
        comboBox.setFixedWidth(width)
    if style:
        comboBox.setStyleSheet(style)
    if items:
        for item in items:
            comboBox.addItem(item)
    if current_changed:
        comboBox.currentTextChanged.connect(current_changed)
    if invisible:
        comboBox.setVisible(False)
    if tooltip:
        TooltipManager(comboBox, tooltip)

    return comboBox


def create_checkBox(text=None, width=None, checked=False, state_changed=None, tooltip=""):
    """
    创建复选框
    """
    checkBox = QCheckBox(text)
    if width:
        checkBox.setFixedWidth(width)
    checkBox.setChecked(checked)
    if state_changed:
        checkBox.stateChanged.connect(state_changed)
    if tooltip:
        TooltipManager(checkBox, tooltip)

    return checkBox


def create_textEdit(default_text="", width=None, max_height=None, read_only=False, line_wrap_mode=None, tooltip=""):
    """
    创建文本编辑框
    """
    textEdit = QTextEdit(default_text)
    if width:
        textEdit.setFixedWidth(width)
    if max_height:
        textEdit.setMaximumHeight(max_height)
    if read_only:
        textEdit.setReadOnly(True)
    if line_wrap_mode:
        textEdit.setLineWrapMode(line_wrap_mode)
    if tooltip:
        TooltipManager(textEdit, tooltip)
    return textEdit


def create_textBrowser(width=None, height=None, tooltip=""):
    """
    创建文本浏览器
    """
    textBrowser = QTextBrowser()
    if width:
        textBrowser.setFixedWidth(width)
    if height:
        textBrowser.setFixedHeight(height)
    if tooltip:
        TooltipManager(textBrowser, tooltip)

    return textBrowser


def add_widgets_to_gridlayout(layout, widgets):
    """
    将控件添加至 GridLayout 布局
    """
    for widget_info in widgets:
        widget = widget_info[0]
        row = widget_info[1]
        col = widget_info[2]
        row_span = widget_info[3] if len(widget_info) > 3 else 1
        col_span = widget_info[4] if len(widget_info) > 4 else 1
        alignment = widget_info[5] if len(widget_info) > 5 else Qt.Alignment()

        layout.addWidget(widget, row, col, row_span, col_span, alignment)


def add_widgets_to_vhboxlayout(layout, widgets):
    """
    将控件添加至 H/VBoxLayout 布局
    """
    for widget_info in widgets:
        widget = widget_info[0]
        stretch = widget_info[1] if len(widget_info) > 1 else 0
        alignment = widget_info[2] if len(widget_info) > 2 else Qt.Alignment()
        layout.addWidget(widget, stretch, alignment)


def open_local_url(url):
    """
    打开本地链接
    """
    url = url.replace("\\", "/")
    url = QUrl.fromLocalFile(url)
    QDesktopServices.openUrl(url)



"""
Pyinstaller build 脚本，一键将 Python 项目根据指定设置打包成 exe 文件，并复制到指定文件夹中
"""


import os
import shutil
import subprocess


MAIN_FILE = 'main.py'  # 主文件
APP_NAME = 'dvatioff-音频工具集'  # 应用名称
TARGET_PATH = r"dvatioff-音频工具集"  # 目标路径
NOCONSOLE = False  # 是否显示控制台
ONEFILE = False  # 是否打包成一个文件
WINDOWED = False  # 是否隐藏控制台
OTHER_DEPENDENCY = True  # 是否复制其他依赖文件，由于 Pyinstaller 打包对部分胞体存在依赖丢失的问题，每次打包需要手动复制这些依赖文件；这里提前将这些依赖文件复制到 dependency 文件夹中，build 成功后会自动复制到打包文件夹中


def build_main(main_file,  app_name='main', noconsole=True, windowed=False, onefile=False):
    if os.path.exists('../../build'):
        shutil.rmtree('../../build')
    if os.path.exists('../../dist'):
        shutil.rmtree('../../dist')

    command = ['pyinstaller', main_file, '--clean']
    if app_name:
        command.append(f'--name={app_name}')
    if noconsole:
        command.append('--noconsole')
    if windowed:
        command.append('--windowed')
    if onefile:
        command.append('--onefile')

    try:
        subprocess.run(command, check=True)
        print("Build succeeded")
        return True
    except subprocess.CalledProcessError as e:
        print(f"Build failed: {e}")
        return False

def copy_app(target_path, onefile=False, dependency=False):
    dist_path = 'dist' if onefile else f'dist/{APP_NAME}'
    if not os.path.exists(target_path):
        os.makedirs(target_path)

    if not onefile:
        internal_folder = os.path.join(dist_path, '_internal')
        if os.path.exists(internal_folder):
            if dependency:
                dependency_folder = r'dependency'
                # 将dependency文件夹中的所有子文件夹复制到_internal文件夹中
                for folder in os.listdir(dependency_folder):
                    folder_path = os.path.join(dependency_folder, folder)
                    target_folder_path = os.path.join(internal_folder, folder)
                    if os.path.exists(target_folder_path):
                        shutil.rmtree(target_folder_path)
                    shutil.copytree(folder_path, target_folder_path)
                    print(f"Copied {folder} folder to _internal folder")

            target_internal_folder = os.path.join(target_path, '_internal')
            if os.path.exists(target_internal_folder):
                shutil.rmtree(target_internal_folder)
            shutil.copytree(internal_folder, target_internal_folder)
            print(f"Copied _internal folder to {target_path}")

    for file_name in os.listdir(dist_path):
        if file_name.endswith('.exe'):
            source_file = os.path.join(dist_path, file_name)
            target_file = os.path.join(target_path, file_name)
            shutil.copy2(source_file, target_file)
            print(f"Copied {file_name} to {target_path}")


def main():
    if build_main(MAIN_FILE, app_name=APP_NAME, noconsole=NOCONSOLE, windowed=WINDOWED, onefile=ONEFILE):
        copy_app(TARGET_PATH, onefile=ONEFILE, dependency=OTHER_DEPENDENCY)


if __name__ == '__main__':
    main()



import azure.cognitiveservices.speech as speechsdk
from dvatioff_audio.utils import retry


# Azure TTS 语音模板
VOICE_TEMPLATE = {
    "Chinese": "zh-CN-XiaoxiaoNeural",
    "English": "en-US-JennyNeural",
    "Korean": "ko-KR-SunHiNeural",
    "Japanese": "ja-JP-NanamiNeural",
}


@retry(Exception, tries=10, delay=1)
def text2speech(text, language, path, subscription, region):
    try:
        # This example requires environment variables named "SPEECH_KEY" and "SPEECH_REGION"
        speech_config = speechsdk.SpeechConfig(subscription=subscription, region=region)
        audio_config = speechsdk.audio.AudioOutputConfig(use_default_speaker=False, filename=path)

        # The language of the voice that speaks.
        speech_config.speech_synthesis_voice_name = VOICE_TEMPLATE[language]
        speech_synthesizer = speechsdk.SpeechSynthesizer(speech_config=speech_config, audio_config=audio_config)
        # Get text from the console and synthesize to the default speaker.
        speech_synthesis_result = speech_synthesizer.speak_text_async(text).get()

        if speech_synthesis_result.reason == speechsdk.ResultReason.SynthesizingAudioCompleted:
            print("AI占位语音生成中： [{}]".format(text))
        elif speech_synthesis_result.reason == speechsdk.ResultReason.Canceled:
            cancellation_details = speech_synthesis_result.cancellation_details
            print("Speech synthesis canceled: {}".format(cancellation_details.reason))
            if cancellation_details.reason == speechsdk.CancellationReason.Error:
                if cancellation_details.error_details:
                    print("Error details: {}".format(cancellation_details.error_details))
                    print("Did you set the speech resource key and region values?")
    except Exception as e:
        print(f"调用 Azure TTS 接口时出错: {e}")
        raise



STATUS_GREEN_COLOR_1 = '#739072'
BLUE_COLOR_1 = '#596FB7'
YELLOW_COLOR_1 = '#F3CA52'

FONT = """
    font-family: 'Microsoft YaHei';
"""

LINEEIDT_FONT = """
    font-family: 'Microsoft YaHei;
    font-size: 12px;
"""

BUTTON_STYLE = """
            QPushButton {
                font-family: "Microsoft YaHei";
                background-color: #7895B2;
                border: 2px solid #7895B2;
                color: #FFFFFF;
            }

            QPushButton:hover {
                font-family: "Microsoft YaHei";
                background-color: #9BB0C1;
                border: 2px solid #9BB0C1;
            }

            QPushButton:disabled {
                font-family: "Microsoft YaHei";
                background-color: #d4dbe4;
                border: 2px solid #d4dbe4;
                color: #A0A0A0;
            }

            QPushButton:pressed {
                font-family: "Microsoft YaHei";
                border: 4px solid #5a7c9d;
            }
        """

BUTTON_STYLE_ORANGE = """
            QPushButton {
                font-family: "Microsoft YaHei";
                background-color: #ED9455;
                border: 2px solid #ED9455;
                color: #FFFFFF;
            }

            QPushButton:hover {
                font-family: "Microsoft YaHei";
                background-color: #FFBB70;
                border: 2px solid #FFBB70;
            }

            QPushButton:disabled {
                font-family: "Microsoft YaHei";
                background-color: #f7ceb1;
                border: 2px solid #f7ceb1;
                color: #A0A0A0;
            }

            QPushButton:pressed {
                font-family: "Microsoft YaHei";
                border: 2px solid #de6a18;
            }
        """

BUTTON_STYLE_ORANGE_TRANSPARENT = """
            QPushButton {
                font-family: "Microsoft YaHei";
                background-color: transparent; /* 修改为透明背景 */
                border: 2px solid #ED9455;
                color: #ED9455; /* 修改为按钮文字的颜色 */
            }

            QPushButton:hover {
                font-family: "Microsoft YaHei";
                background-color: transparent; /* 修改为透明背景 */
                border: 2px solid #FFBB70;
                color: #FFBB70; /* 修改为按钮文字的颜色 */
            }

            QPushButton:disabled {
                font-family: "Microsoft YaHei";
                background-color: transparent; /* 修改为透明背景 */
                border: 2px solid #f7ceb1;
                color: #f7ceb1; /* 修改为按钮文字的颜色 */
            }

            QPushButton:pressed {
                font-family: "Microsoft YaHei";
                border: 2px solid #de6a18;
            }
        """

BUTTON_STYLE_GREEN = """
            QPushButton {
                font-family: "Microsoft YaHei";
                background-color: #739072;
                border: 2px solid #739072;
                color: #FFFFFF;
            }

            QPushButton:hover {
                font-family: "Microsoft YaHei";
                background-color: #86A789;
                border: 2px solid #86A789;
            }

            QPushButton:disabled {
                font-family: "Microsoft YaHei";
                background-color: #E7F0E2;
                border: 2px solid #E7F0E2;
                color: #A0A0A0;
            }

            QPushButton:pressed {
                font-family: "Microsoft YaHei";
                border: 4px solid #4F6F52;
            }
        """

BUTTON_STYLE_GREEN_TRANSPARENT = """
            QPushButton {
                font-family: "Microsoft YaHei";
                background-color: transparent; /* 修改为透明背景 */
                border: 2px solid #7895B2;
                color: #7895B2; /* 修改为按钮文字的颜色 */
            }

            QPushButton:hover {
                font-family: "Microsoft YaHei";
                background-color: transparent; /* 修改为透明背景 */
                border: 2px solid #9BB0C1;
                color: #9BB0C1; /* 修改为按钮文字的颜色 */
            }

            QPushButton:disabled {
                font-family: "Microsoft YaHei";
                background-color: transparent; /* 修改为透明背景 */
                border: 2px solid #d4dbe4;
                color: #d4dbe4; /* 修改为按钮文字的颜色 */
            }

            QPushButton:pressed {
                font-family: "Microsoft YaHei";
                border: 4px solid #5a7c9d;
            }
        """

BUTTON_STYLE_PINK = """
            QPushButton {
                font-family: "Microsoft YaHei";
                background-color: #E195AB;
                border: 2px solid #E195AB;
                color: #FFFFFF;
            }

            QPushButton:hover {
                font-family: "Microsoft YaHei";
                background-color: #FFCCE1;
                border: 2px solid #FFCCE1;
            }

            QPushButton:disabled {
                font-family: "Microsoft YaHei";
                background-color: #F2F9FF;
                border: 2px solid #F2F9FF;
                color: #A0A0A0;
            }

            QPushButton:pressed {
                font-family: "Microsoft YaHei";
                border: 4px solid #d87693;
            }
        """

LABEL_STYLE = """
            font-family: "Microsoft YaHei";
            font-weight: bold;
            font-size: 10pt;
        """

LABEL_STYLE_STATUS = """
            font-family: "Microsoft YaHei";
            font-size: 9pt;
        """

LABEL_STYLE_NORMAL = """
            font-family: "Microsoft YaHei";
            font-size: 10pt;
        """

LABEL_STYLE_TITLE = """
            font-family: "Microsoft YaHei";
            font-weight: bold;
            font-size: 18pt;
        """
LABEL_STYLE_SUBTITLE = """
            font-family: "Microsoft YaHei";
            font-weight: bold;
            font-size: 12pt;
        """

LABEL_STYLE_SIZE15 = """
            font-family: "Microsoft YaHei";
            font-weight: bold;
            font-size: 15pt;
        """

TEXT_BOLD = """
            font-family: "Microsoft YaHei";
            font-size: 12px;
            font-weight: bold;
"""

LABEL_STATUS_GREEN = """
            background-color: #4bb762;
            padding: 5px;
"""

LABEL_STATUS_RED = """
            background-color: #BF3131;
            padding: 5px;
"""
LABEL_STATUS_YELLOW = """
            background-color: #FFB534;
            padding: 5px;
"""

COLOR = {
    "black": "#000000",
    "blue": "#40A2D8",
    "green": "#65B741",
    "red": "#BF3131",
    "orange": "#FFB534",
    "pink": "#FF90BC"
}

SCROLL_AREA_ORANGE = """
            QScrollArea{
                border: 2px solid #f7ceb1;
            }
        """



"""
此脚本中的类用于启动一个子线程，实时获取当前 Wwise 工程选中的对象，并通过 Signal 信号将对象信息传递给主线程
"""


import time
from PySide6.QtCore import Signal, QRunnable, QObject
import threading
from dvatioff_audio.waapi.waapi_utils import get_selected_objects


class SignalSelectedObjects(QObject):
    signal = Signal(list)


class WwiseObjectSelectWorker(QRunnable):
    def __init__(self, client):
        super().__init__()

        self.client = client
        self.running = threading.Event()
        self.running.set()
        self.worker_thread = None

        self.signal_selected_objects = SignalSelectedObjects()

    def run(self):
        self.worker_thread = threading.Thread(target=self.run_in_thread)
        self.worker_thread.start()

    def run_in_thread(self):
        while self.running.is_set():
            if not self.client or not self.client.is_connected():
                self.stop()
                break
            self.get_selected_objects()
            time.sleep(0.5)

    def get_selected_objects(self):
        """
        获取当前 Wwise 工程选中的对象
        """
        if self.client and self.client.is_connected():
            result = get_selected_objects(self.client)
            if result:
                selected_objects = result['objects']
                self.signal_selected_objects.signal.emit(selected_objects)
            else:
                self.signal_selected_objects.signal.emit([])
        else:
            self.signal_selected_objects.signal.emit([])

    def stop(self):
        self.running.clear()
        if self.worker_thread:
            self.worker_thread.join()
            self.worker_thread = None



# Wwise Connection
UNCONNECTED = 0
CONNECTED = 1
WRONG_CONNECTED = 2

# Page Index
INDEX_PAGE_SFX_IMPORT = 0
INDEX_PAGE_VO_IMPORT = 1
INDEX_MULTILANGUAGE_SILENCE = -1

# Path
PATH_ICON_PROGRAM = 'Icons\\Icon_Program.png'
PATH_ICON_WARNING = 'Icons\\Icon_Warning.png'
PATH_ICON_NOTIFICATION = 'Icons\\Icon_Notification.png'
PATH_ICON_EDIT = 'Icons\\Icon_Edit.png'
PATH_ICON_SUCCESS = 'Icons\\Icon_Success.png'
PATH_ICON_VFY = 'Icons\\Icon_VFY.png'
PATH_ICON_WWISE = {
    "Sound": "Icons/Wwise/Wwise_Sound.png",
    "SwitchContainer": "Icons/Wwise/Wwise_SwitchContainer.png",
    "RandomContainer": "Icons/Wwise/Wwise_RandomContainer.png",
    "SequenceContainer": "Icons/Wwise/Wwise_SequenceContainer.png",
    "BlendContainer": "Icons/Wwise/Wwise_BlendContainer.png"
}

# Wwise Info
WWISE_PROJECT_NAME = ''
WWISE_VERSION = 'Wwise 20xx.x.xx'
WWISE_LANGUAGE = ['Chinese', 'English', 'Korean', 'Japanese']
WWISE_VO_IMPORT_TYPE = ['AI 占位资源', '正式资源']


import azure.cognitiveservices.speech as speechsdk
import time
import threading
from dvatioff_audio.utils import retry


# Azure STT 支持的语言
LANGUAGE = {
    "Chinese": "zh-CN",
    'English': "en-US",
    "Japanese": "ja-JP",
    'Korean': "ko-KR",
}


@retry(Exception, tries=10, delay=5)
def speech_to_text(audio_file, language, subscription, region):
    done = False
    text = ""

    def stop_cb(evt):
        nonlocal done
        print('CLOSING on {}'.format(evt))
        # 在新线程中执行 stop_continuous_recognition 以避免阻塞
        threading.Thread(target=speech_recognizer.stop_continuous_recognition).start()
        done = True

    def handle_result(evt):
        nonlocal text
        text += evt.result.text
        print('RECOGNIZED:', evt.result.text)

    speech_config = speechsdk.SpeechConfig(subscription=subscription, region=region)
    speech_config.speech_recognition_language = LANGUAGE[language]
    audio_config = speechsdk.audio.AudioConfig(filename=audio_file)
    speech_recognizer = speechsdk.SpeechRecognizer(speech_config=speech_config, audio_config=audio_config)

    speech_recognizer.recognized.connect(handle_result)
    speech_recognizer.session_stopped.connect(stop_cb)
    speech_recognizer.canceled.connect(stop_cb)

    try:
        speech_recognizer.start_continuous_recognition()
        while not done:
            time.sleep(0.1)

        if language == 'English':
            lower_text = text.lower()
            return lower_text

        return text
    except Exception as e:
        print(f"调用 Azure STT 接口时出错: {e}")
        return ""



from dvatioff_audio.utils import minimize_console


def main():
    print("工具初始化中...", flush=True)

    from PySide6.QtCore import QTimer
    from PySide6.QtWidgets import QApplication
    from qt_material import apply_stylesheet
    from dvatioff_audio.examples.main_window_framework import MainWindow

    app = QApplication()
    QTimer.singleShot(100, minimize_console)
    window = MainWindow()
    apply_stylesheet(app, theme='theme_ReCat.xml', invert_secondary=True)
    window.show()

    app.exec()


if __name__ == '__main__':
    main()



from PySide6.QtCore import QSettings, QThreadPool, Slot
from PySide6.QtGui import QIcon
from PySide6.QtWidgets import QMainWindow, QStackedWidget, QWidget, QFileDialog, QStatusBar
from dvatioff_audio.gui.gui_utils import disable_button, create_label, create_button, create_textBrowser, add_widgets_to_gridlayout, create_layout, enable_button
import dvatioff_audio.gui.gui_css as css
from dvatioff_audio.waapi.waapi_connector import WaapiConnectWorker
from functools import partial
from dvatioff_audio.waapi.waapi_object_selector import WwiseObjectSelectWorker
from dvatioff_audio.utils import open_url
import dvatioff_audio.globals as g


class MainWindow(QMainWindow):
    def __init__(self, ):
        super().__init__()

        self.stacked_widget = QStackedWidget()
        self.setCentralWidget(self.stacked_widget)
        self.setting = QSettings("dvatiOFF", "导入选择")
        self.setWindowIcon(QIcon(""))

        self.threadpool = QThreadPool()  # 线程池
        self.thread_waapi_connect = None
        self.worker_waapi_connect = None
        self.worker_get_wwise_selected_objects = None

        self.waapi_client = None  # WAAPI 客户端
        self.connect_status = False  # WAAPI 的连接状态
        self.connected = g.UNCONNECTED  # WAAPI 详细的连接状态，分为 未连接/已连接/连接至错误的 Wwise 工程或版本 三种状态
        self.wwise_project_name = None  # Wwise 工程名称
        self.wwise_version = None  # Wwise 版本
        self.statusBar = None  # 状态栏
        self.status_message = None  # 状态栏信息
        self.status_color = None  # 状态栏颜色（表示连接状态）

        self.confirm_window = None  # 各类操作执行后弹出的确认窗口

        self.wwise_selected_objects = []  # Wwise 中选中的对象
        self.wwise_selected_objects_legal = []  # Wwise 中选中的合法对象
        self.buttons_all = []  # 所有 GUI 按钮
        self.buttons_connect_activate = []  # 连接至 Wwise 后激活的按钮

        self.page_sfx_import = None  # 音效导入页面

        self.init_UI()

    def init_UI(self):
        """
        初始化UI
        """
        self.setWindowTitle("dvatiOFF-音频工具集")
        self.setFixedSize(1000, 600)
        self.init_status_bar()
        self.create_page_sfx_import()
        self.setup_menu()
        self.execute_waapi_connect_worker()

    def setup_menu(self):
        """
        设置菜单栏
        """
        menu_bar = self.menuBar()
        menu_bar.setStyleSheet(css.FONT_PURE)

        menu_vo = menu_bar.addMenu("语音")
        action_vo_import = menu_vo.addAction("语音占位导入")
        action_vo_import.triggered.connect(partial(self.switch_page, 0))

        menu_audio = menu_bar.addMenu("音效")
        action_audio_3p = menu_audio.addAction("音效导入")
        action_audio_3p.triggered.connect(partial(self.switch_page, 0))
        menu_audio.addAction(action_audio_3p)

        menu_help = menu_bar.addMenu("帮助")
        action_help = menu_help.addAction("音效命名规则")
        action_help.triggered.connect(partial(open_url, ""))
        menu_help.addAction(action_help)

    def remove_widgets(self):
        """
        移除所有的页面
        """
        self.stacked_widget.removeWidget(self.page_sfx_import)
        self.page_sfx_import.deleteLater()

    def refresh_UI(self):
        """
        刷新页面显示
        """
        self.buttons_all = []
        self.buttons_connect_activate = []
        self.remove_widgets()
        self.create_page_sfx_import()
        if not self.connect_status:
            self.disable_buttons()

    def switch_page(self, index):
        """
        切换页面
        :param index: 页面索引(见全局变量定义)
        """
        if index == g.INDEX_PAGE_SFX_IMPORT:
            self.setFixedSize(1000, 400)

        self.refresh_UI()
        self.stacked_widget.setCurrentIndex(index)

    # START ------------------ 状态栏相关 ------------------

    def init_status_bar(self):
        """
        初始化状态栏
        """
        self.statusBar = QStatusBar()
        self.setStatusBar(self.statusBar)
        self.status_message = create_label("未连接至 Wwise Authoring", css.LABEL_STYLE_STATUS)
        self.statusBar.addPermanentWidget(self.status_message)
        self.status_color = create_label("  ", css.LABEL_STATUS_RED, 16, 16)
        self.status_color.setStyleSheet(css.LABEL_STATUS_RED)
        self.statusBar.addPermanentWidget(self.status_color)

    def execute_waapi_connect_worker(self):
        self.worker_waapi_connect = WaapiConnectWorker()
        self.worker_waapi_connect.signal_connection_status.signal.connect(self.status_changed)
        self.worker_waapi_connect.signal_waapi_client.signal.connect(self.get_waapi_client)
        self.worker_waapi_connect.signal_project_name.signal.connect(self.get_wwise_project_name_info)

        self.threadpool.start(self.worker_waapi_connect)

    @Slot(bool)
    def status_changed(self, status):
        """
        连接状态改变时更新状态栏显示
        """
        self.connect_status = status
        if status:
            if self.wwise_project_name == g.WWISE_PROJECT_NAME and self.wwise_version == g.WWISE_VERSION:
                self.status_color.setStyleSheet(css.LABEL_STATUS_GREEN)
                self.status_message.setText(f"{self.wwise_project_name} - {self.wwise_version} 已连接")
                self.enable_buttons()
                self.connected = g.CONNECTED
            elif self.wwise_project_name == g.WWISE_PROJECT_NAME and self.wwise_version != g.WWISE_VERSION:
                self.status_color.setStyleSheet(css.LABEL_STATUS_YELLOW)
                self.status_message.setText(f"{self.wwise_project_name} - {self.wwise_version} 已连接 (Wwise 版本不匹配)")
                self.disable_buttons()
                self.connected = g.WRONG_CONNECTED
            elif self.wwise_project_name != g.WWISE_PROJECT_NAME and self.wwise_version == g.WWISE_VERSION:
                self.status_color.setStyleSheet(css.LABEL_STATUS_YELLOW)
                self.status_message.setText(f"{self.wwise_project_name} - {self.wwise_version} 已连接 (Wwise 工程不匹配)")
                self.disable_buttons()
                self.connected = g.WRONG_CONNECTED
            else:
                self.status_color.setStyleSheet(css.LABEL_STATUS_YELLOW)
                self.status_message.setText(f"{self.wwise_project_name} - {self.wwise_version} 已连接 (Wwise 工程和版本不匹配)")
                self.disable_buttons()
                self.connected = g.WRONG_CONNECTED
        else:
            self.status_color.setStyleSheet(css.LABEL_STATUS_RED)
            self.status_message.setText("未连接至 Wwise Authoring")
            self.disable_buttons()
            self.connected = g.UNCONNECTED

        if self.connected != g.CONNECTED and self.worker_get_wwise_selected_objects:
            self.worker_get_wwise_selected_objects.stop()
            self.worker_get_wwise_selected_objects = None

    @Slot(object)
    def get_waapi_client(self, client):
        self.waapi_client = client

    def execute_get_wwise_objects_worker(self):
        self.worker_get_wwise_selected_objects = WwiseObjectSelectWorker(self.waapi_client)
        self.worker_get_wwise_selected_objects.signal_selected_objects.signal.connect(self.get_selected_objects)
        self.threadpool.start(self.worker_get_wwise_selected_objects)

    @Slot(list)
    def get_selected_objects(self, selected_objects):
        if len(selected_objects) != len(self.wwise_selected_objects) or any(a != b for a, b in zip(selected_objects, self.wwise_selected_objects)):
            self.wwise_selected_objects = selected_objects
            # self.add_scroll_content(self.layout_scroll_event_soundbank)

    @Slot(str, str)
    def get_wwise_project_name_info(self, project_name, wwise_version):
        self.wwise_project_name = project_name
        self.wwise_version = wwise_version

    # END ------------------ 状态栏相关 ------------------

    # START ------------------ 页面相关 ------------------

    # ================== 音效占位导入 ==================
    def create_page_sfx_import(self):
        """
        创建音效导入页面
        """
        self.page_sfx_import = QWidget()
        self.stacked_widget.addWidget(self.page_sfx_import)

        layout = create_layout("grid", self.page_sfx_import, margin=(50, 50, 50, 50))

        # ================== 元素添加 ==================
        title = create_label("音效占位导入", css.LABEL_STYLE_TITLE, height=80)

        label_select_excel = create_label("选择 Excel 文件: ", css.LABEL_STYLE, width=100)
        textBrowser_select_execl = create_textBrowser(width=700, height=36)
        button_select_excel = create_button("选择", css.BUTTON_STYLE_PINK, tooltip="选择导入音效所需的 Excel 文件")
        button_name_vfy = create_button("音频文件名校验", css.BUTTON_STYLE_PINK, enabled=False)
        button_import = create_button("导入", css.BUTTON_STYLE_PINK, enabled=False)

        # ================== 事件绑定 ==================
        button_select_excel.clicked.connect(partial(self.open_file_dialog, textBrowser_select_execl, button_name_vfy, button_import))

        # ================== 页面布局 ==================
        add_widgets_to_gridlayout(layout, [
            [title, 0, 0, 1, 3],
            [label_select_excel, 1, 0], [textBrowser_select_execl, 1, 1], [button_select_excel, 1, 2],
            [button_name_vfy, 2, 0, 1, 3],
            [button_import, 3, 0, 1, 3]
        ])

    # END ------------------ 页面相关 ------------------

    def disable_buttons(self):
        """
        禁用所有按钮
        """
        for button in self.buttons_all:
            button.setDisabled(True)

    def enable_buttons(self):
        """
        启用所有按钮
        """
        for button in self.buttons_connect_activate:
            button.setDisabled(False)

    def open_file_dialog(self, textBrowser, button_name_vfy, button_import):
        last_file_path = self.setting.value("last_file_path", "")
        file_path, _ = QFileDialog.getOpenFileName(self, '选择文件', last_file_path, "Excel 文件 (*.xlsx)")

        if file_path:
            enable_button(button_name_vfy)
            if file_path != textBrowser.toPlainText():
                disable_button(button_import)
                textBrowser.setText(file_path)
                self.setting.setValue("last_folder_path", file_path)

    def open_confirm_window(self, **kwargs):
        """
        打开确认窗口
        """
        task = kwargs.get("task")
        if task == g.INDEX_PAGE_VO_IMPORT:
            title = "确认"
            message = "是否将正式资源导入 Wwise?"

    def execute_task(self, **kwargs):
        """
        关闭确认窗口
        """
        task = kwargs.get("task")




import os
from dvatioff_audio.utils import retry
from lxml import etree
from tqdm import tqdm


@retry(Exception, tries=10, delay=1)
def get_connected_project_name_info(client):
    """
    获取当前 Wwise 工程名称
    """
    try:
        result_wwise_version = client.call("ak.wwise.core.getInfo")
        result_project_name = client.call("ak.wwise.core.object.get", {"from": {"ofType": ["Project"]}})

        if 'version' in result_wwise_version and 'return' in result_project_name:
            wwise_version = result_wwise_version['version']['displayName'].replace("v", "Wwise ")
            project_name = result_project_name['return'][0]['name']
            print(f"成功获取项目信息: {project_name} - {wwise_version}")
            return project_name, wwise_version
        else:
            raise ValueError("获取项目信息失败： Wwise Authoring 可能正在初始化...")
    except Exception as e:
        print(f"调用 ak.wwise.core.getProjectInfo 时出错: {e}")
        raise


@retry(Exception, tries=10, delay=1)
def get_object(client, object_path):
    """
    获取 Wwise 对象信息
    """
    try:
        args = {
            "from": {
                "path": [object_path]
            }
        }
        result = client.call("ak.wwise.core.object.get", args)
        return result
    except Exception as e:
        print(f"调用 ak.wwise.core.object.get 时出错: {e}")
        return None


@retry(Exception, tries=10, delay=1)
def get_object_waql(client, waql):
    """
    使用 WAQL 获取 Wwise 对象信息
    """
    try:
        args = {
            "waql": waql
        }
        result = client.call("ak.wwise.core.object.get", args)
        return result
    except Exception as e:
        print(f"调用 ak.wwise.core.object.get 时出错: {e}")
        return None


@retry(Exception, tries=10, delay=1)
def get_selected_objects(client):
    """
    获取当前 Wwise 工程选中的对象
    """
    try:
        opts = {
            "return": [
                "id",
                "name",
                "type",
                "path",
                "RandomOrSequence",
            ]
        }
        result = client.call("ak.wwise.ui.getSelectedObjects", options=opts)
        if 'objects' in result:
            # 分离 RandomOrSequence 类型
            for obj in result['objects']:
                if 'RandomOrSequence' in obj:
                    obj['type'] = 'SequenceContainer' if obj['RandomOrSequence'] == 0 else 'RandomContainer'
            return result
        else:
            return None
    except Exception as e:
        print(f"调用 ak.wwise.ui.getSelectedObjects 时出错: {e}")
        return None


@retry(Exception, tries=10, delay=1)
def create_object(client, object_type, parent_path, object_name, on_name_conflict="merge", children=None, output_bus=None, makeup_gain=None, priority=None, enable_hdr_envelope=None):
    """
    创建 Wwise 对象
    """
    try:
        args = {
            "parent": parent_path,
            "type": object_type,
            "name": object_name,
            "onNameConflict": on_name_conflict,
        }
        if children:
            args["children"] = children
        if output_bus:
            args["@OutputBus"] = output_bus
            args['@OverrideOutput'] = True
        if makeup_gain:
            args["@MakeUpGain"] = makeup_gain
        if priority:
            args["@Priority"] = priority
            args["@OverridePriority"] = True
        if enable_hdr_envelope:
            args["@OverrideHdrEnvelope"] = True
            args["@HdrEnableEnvelope"] = True
            args["@HdrEnvelopeSensitivity"] = 9
        result = client.call("ak.wwise.core.object.create", args)
        return result
    except Exception as e:
        print(f"调用 ak.wwise.core.object.create 时出错: {e}")
        return None


@retry(Exception, tries=10, delay=1)
def set_object(client, objects):
    """
    设置 Wwise 对象属性
    """
    try:
        args = {
            "objects": []
        }
        for obj in objects:
            arg = {}
            for key, value in obj.items():
                arg[key] = value
            args["objects"].append(arg)
        result = client.call("ak.wwise.core.object.set", args)
        return result
    except Exception as e:
        print(f"调用 ak.wwise.core.object.setProperty 时出错: {e}")
        return None


@retry(Exception, tries=10, delay=1)
def set_soundbank_inclusion(client, soundbank, obj, operation="add", inclusions_filter=None):
    """
    设置 SoundBank 包含的对象
    """
    try:
        if inclusions_filter is None:
            inclusions_filter = [
                "events",
                "structures",
                "media"
            ]
        args = {
            "soundbank": soundbank,
            "operation": operation,
            "inclusions": [
                {
                    "object": obj,
                    "filter": inclusions_filter
                }
            ]
        }
        result = client.call("ak.wwise.core.soundbank.setInclusions", args)
        return result
    except Exception as e:
        print(f"调用 ak.wwise.core.soundbank.setInclusions 时出错: {e}")
        return None


@retry(Exception, tries=10, delay=1)
def import_audio_file(client, **kwargs):
    """
    导入音频文件到 Wwise
    """
    try:
        args = {
            "importOperation": "useExisting",
            "default": {
                "importLanguage": kwargs.get("importLanguage", "")
            },
            "imports": [
                {
                    "audioFile": kwargs.get("audioFile", ""),
                    "originalsSubFolder": kwargs.get("originalsSubFolder", ""),
                    "objectPath": kwargs.get("objectPath", "")
                }
            ]
        }
        if kwargs.get("isLoopingEnabled", False):
            args["imports"][0]["@IsLoopingEnabled"] = True
        if kwargs.get("notes", ""):
            args["imports"][0]["notes"] = kwargs.get("notes", "")
        result = client.call("ak.wwise.core.audio.import", args)
        return result
    except Exception as e:
        print(f"调用 ak.wwise.core.audio.import 时出错: {e}")
        return None


@retry(Exception, tries=10, delay=1)
def get_wwise_originals_sfx_path(client):
    """
    获取 Wwise Originals SFX 文件夹的路径
    """
    try:
        result = client.call("ak.wwise.core.getProjectInfo")
        if 'directories' in result:
            originals_path = result['directories']['originals']
            return os.path.join(originals_path, "SFX")
        else:
            raise ValueError("获取 Wwise Originals SFX 文件夹路径失败")
    except Exception as e:
        print(f"调用 ak.wwise.core.getProjectInfo 时出错: {e}")
        return None


@retry(Exception, tries=10, delay=1)
def get_wwise_originals_path(client):
    """
    获取 Wwise Originals 文件夹路径
    """
    try:
        result = client.call("ak.wwise.core.getProjectInfo")
        if 'directories' in result:
            originals_path = result['directories']['originals']
            return originals_path
        else:
            raise ValueError("获取 Wwise Originals 文件夹路径失败")
    except Exception as e:
        print(f"调用 ak.wwise.core.getProjectInfo 时出错: {e}")
        return None


@retry(Exception, tries=10, delay=1)
def add_switch_assignment(client, child, stateOrSwitch):
    """
    添加 Switch Assignment
    """
    try:
        args = {
            "child": child,
            "stateOrSwitch": stateOrSwitch
        }
        result = client.call("ak.wwise.core.switchContainer.addAssignment", args)
        return result
    except Exception as e:
        print(f"调用 ak.wwise.core.switchContainer.addAssignment 时出错: {e}")
        return None


@retry(Exception, tries=10, delay=1)
def copy_object(client, obj, parent, on_name_conflict="rename"):
    """
    复制 Wwise 对象
    """
    try:
        args = {
            "object": obj,
            "parent": parent,
            "onNameConflict": on_name_conflict
        }
        result = client.call("ak.wwise.core.object.copy", args)
        return result
    except Exception as e:
        print(f"调用 ak.wwise.core.object.copy 时出错: {e}")
        return None


@retry(Exception, tries=10, delay=1)
def move_object(client, obj, parent, on_name_conflict="rename"):
    """
    移动 Wwise 对象
    """
    try:
        args = {
            "object": obj,
            "parent": parent,
            "onNameConflict": on_name_conflict
        }
        result = client.call("ak.wwise.core.object.move", args)
        return result
    except Exception as e:
        print(f"调用 ak.wwise.core.object.move 时出错: {e}")
        return None


@retry(Exception, tries=10, delay=1)
def set_name(client, obj, name):
    """
    设置 Wwise 对象名称
    """
    try:
        args = {
            "object": obj,
            "value": name
        }
        result = client.call("ak.wwise.core.object.setName", args)
        return result
    except Exception as e:
        print(f"调用 ak.wwise.core.object.setName 时出错: {e}")
        return None


@retry(Exception, tries=10, delay=1)
def set_notes(client, obj, notes):
    """
    设置 Wwise 对象备注
    """
    try:
        args = {
            "object": obj,
            "value": notes
        }
        result = client.call("ak.wwise.core.object.setNotes", args)
        return result
    except Exception as e:
        print(f"调用 ak.wwise.core.object.setNotes 时出错: {e}")
        return None


@retry(Exception, tries=10, delay=1)
def delete_object(client, obj):
    """
    删除 Wwise 对象
    """
    try:
        args = {
            "object": obj
        }
        result = client.call("ak.wwise.core.object.delete", args)
        return result
    except Exception as e:
        print(f"调用 ak.wwise.core.object.delete 时出错: {e}")
        return None


# 下面的函数无需连接 waapi 即可调用
def assign_obj_to_dynamic_event_path(container_dict, dynamic_event_wwu_path, wwu_guid):
    """
    由于 waapi 无法直接为 Dynamic Event 中的各 path 指定对象，因此需要通过解析 Wwise Work Unit 文件来实现
    在指定规则下为 Dynamic Event 中的各 path 指定对象
    """
    tree = etree.parse(dynamic_event_wwu_path)
    root = tree.getroot()
    total_iterations = len(container_dict.keys())
    with tqdm(total=total_iterations, desc="Adding Dynamic Path") as pbar:
        for multi_switch_entry in root.findall('.//MultiSwitchEntry'):
            pbar.update(1)
            state_tuple = ()
            for obj in multi_switch_entry.iterfind('.//ObjectRef'):
                state_tuple += (obj.get('Name'),)

            obj_id = container_dict[state_tuple]
            obj_name = f"Hit_{state_tuple[0]}_{state_tuple[1]}_{state_tuple[2]}_{state_tuple[3]}"
            reference_list = etree.SubElement(multi_switch_entry, "ReferenceList")
            reference = etree.SubElement(reference_list, "Reference", Name='AudioNode')
            object_ref = etree.SubElement(reference, "ObjectRef", Name=obj_name, ID=obj_id, WorkUnitID=wwu_guid)

            object_lists = multi_switch_entry.find('.//ObjectLists')

            index = list(multi_switch_entry).index(object_lists)
            multi_switch_entry.insert(index, reference_list)

    tree.write(dynamic_event_wwu_path, pretty_print=True)




# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="dvatioff-audio",  # 包名，在 PyPI 上必须唯一
    version="0.3.0",  # 版本号
    author="dvatiOFF",
    author_email="dvatioff@gmail.com",
    description="Personal audio utils package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/dvatiOFF/python-audio-utils",  # 可选
    packages=find_packages(),  # 自动查找需要打包的模块
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.10',  # 指定最低 Python 版本
)

# 打包上传步骤：
# python setup.py sdist bdist_wheel
# python -m twine upload dist/*
# API TOKEN = pypi-AgEIcHlwaS5vcmcCJGY3ZGE0ODU1LTRiYWQtNGQzZC05N2JlLWZlZWY1YmIyYmNmOAACFlsxLFsiZHZhdGlvZmYtYXVkaW8iXV0AAixbMixbIjQxM2ZmYjhmLWY0MjAtNDlmZS1iNjgyLTU1N2QxNWViZWViYyJdXQAABiB-AjgJq5CSZ2ffVWhbg_Fu-NDlHkVLraZ21xDaY5UlGA


"""
文本-语音对齐检测窗口框架
"""


import os
import queue
import threading
import time
from PySide6.QtCore import  QRunnable, QObject, Signal, QThreadPool
from PySide6.QtGui import QTextCursor, QIcon
from PySide6.QtWidgets import QWidget
import dvatioff_audio.gui.gui_css as css
from notification_window_framework import NotificationWindow
from dvatioff_audio.voice.voice_utils import text_speech_verify_worker, output_text_speech_verify_result
from dvatioff_audio.gui.gui_utils import create_label, create_button, add_widgets_to_vhboxlayout, create_layout, create_textEdit
from dvatioff_audio.globals import PATH_ICON_VFY


class TextSpeechVerificationWorkerSignals(QObject):
    finished = Signal(dict)


class TextSpeechVerificationWorker(QRunnable):
    def __init__(self, folder_path, text_dict, language):
        super().__init__()
        self.folder_path = folder_path
        self.text_dict = text_dict
        self.language = language
        self.signals = TextSpeechVerificationWorkerSignals()
        self.stop_requested = False

    def run(self):
        update_queue = queue.Queue()
        worker_thread = threading.Thread(target=text_speech_verify_worker,
                                         args=(self.folder_path, self.text_dict, self.language, update_queue))
        worker_thread.start()
        vfy_complete = False

        while True:
            if self.stop_requested:
                break
            try:
                stt_result = update_queue.get_nowait()
                self.signals.finished.emit(stt_result)

                total_index = stt_result.get('total_index', 0)
                total_file_num = stt_result.get('total_file_num', 0)
                if total_index == total_file_num:
                    vfy_complete = True
            except queue.Empty:
                pass
            if vfy_complete:
                break
            time.sleep(0.4)

    def stop(self):
        self.stop_requested = True


class TextSpeechVFYWindow(QWidget):
    def __init__(self, text_dict, folder_path, language, icon_path=PATH_ICON_VFY):
        super().__init__()

        self.threadpool = QThreadPool()
        self.worker = None
        self.notification_window = None
        self.setWindowIcon(QIcon(icon_path))

        self.folder_path = folder_path
        self.text_dict = text_dict
        self.language = language

        self.error_log = ''
        self.error_file_paths = []

        self.layout = create_layout('vbox', self)

        self.label_all_log = create_label('全部日志', style=css.LABEL_STYLE_SIZE15)
        self.label_error_log = create_label('错误日志', style=css.LABEL_STYLE_SIZE15)
        self.label_cur_folder = create_label('<b>当前文件夹:</b> ')
        self.label_cur_detection_progress = create_label('<b>当前文件夹检测进度:</b> ')
        self.label_total_detection_progress = create_label('<b>总检测进度:</b> ')
        self.text_all_log = create_textEdit(read_only=True, max_height=600)
        self.text_error_log = create_textEdit(read_only=True, max_height=400)

        self.output_button = create_button('输出错误文件和日志', style=css.BUTTON_STYLE_PINK, enabled=False)
        self.output_button.clicked.connect(self.output_log_and_files)

        add_widgets_to_vhboxlayout(self.layout, [
            [self.label_all_log],
            [self.label_cur_folder],
            [self.label_cur_detection_progress],
            [self.label_total_detection_progress],
            [self.text_all_log],
            [self.label_error_log],
            [self.text_error_log],
            [self.output_button]
        ])

        self.setWindowTitle("文本-语音对齐检测")
        self.setFixedSize(1400, 1200)
        self.show()

    def closeEvent(self, event):
        if self.worker:
            self.worker.stop()
        event.accept()

    def execute_text_speech_vfy_task(self):
        self.worker = TextSpeechVerificationWorker(self.folder_path, self.text_dict, self.language)
        self.worker.signals.finished.connect(self.text_speech_vfy_ui_update)
        self.threadpool.start(self.worker)

    def text_speech_vfy_ui_update(self, stt_result):
        parent_directory_name = stt_result.get('parent_directory_name', '')
        file_name = stt_result.get('file_name', '')
        file_path = stt_result.get('file_path', '')
        cur_index = stt_result.get('cur_index', 0)
        cur_folder_file_num = stt_result.get('cur_folder_file_num', 0)
        total_index = stt_result.get('total_index', 0)
        total_file_num = stt_result.get('total_file_num', 0)
        true_text = stt_result.get('true_text', '')
        speech_text = stt_result.get('speech_text', '')
        true_pinyin = stt_result.get('true_pinyin', '')
        speech_pinyin = stt_result.get('speech_pinyin', '')
        true_kata = stt_result.get('true_kata', '')
        speech_kata = stt_result.get('speech_kata', '')
        true_kata_romaji = stt_result.get('true_kata_romaji', '')
        speech_kata_romaji = stt_result.get('speech_kata_romaji', '')
        wer = stt_result.get('wer', 0)

        self.label_cur_folder.setText(f'<b>当前文件夹:</b> {parent_directory_name}')
        self.label_cur_detection_progress.setText(f'<b>当前文件夹检测进度:</b> {cur_index}/{cur_folder_file_num}')
        self.label_total_detection_progress.setText(f'<b>总检测进度:</b> {total_index}/{total_file_num}')

        # 保持文本窗口的焦点跟随
        cursor_all_log = self.text_all_log.textCursor()
        cursor_all_log.movePosition(QTextCursor.End)
        self.text_all_log.setTextCursor(cursor_all_log)
        self.text_all_log.ensureCursorVisible()
        cursor_error_log = self.text_error_log.textCursor()
        cursor_error_log.movePosition(QTextCursor.End)
        self.text_error_log.setTextCursor(cursor_error_log)
        self.text_error_log.ensureCursorVisible()

        if true_text == "本句为呼喝声，跳过检测":
            self.text_all_log.textCursor().insertHtml(f"<font color={css.COLOR['black']}><b>当前检测文件:</b> {file_name}</font><br>")
            self.text_all_log.textCursor().insertHtml(f"<font color={css.COLOR['blue']}>{true_text}，跳过检测</font><br><br>")
            return
        elif true_text == "Azure 接口调用失败":
            self.text_all_log.textCursor().insertHtml(f"<font color={css.COLOR['black']}><b>当前检测文件:</b> {file_name}<br>")
            self.text_all_log.textCursor().insertHtml(f"<font color={css.COLOR['pink']}>{true_text}，跳过检测</font><br><br>")
            return

        self.text_all_log.textCursor().insertHtml(f"<font color={css.COLOR['black']}><b>正确文本:</b> {true_text}</font><br>")
        self.text_all_log.textCursor().insertHtml(f"<font color={css.COLOR['black']}><b>识别文本:</b> {speech_text}</font><br>")

        if self.language == "Chinese":
            self.text_all_log.textCursor().insertHtml(f"<font color={css.COLOR['black']}><b>正确拼音:</b> {true_pinyin}</font><br>")
            self.text_all_log.textCursor().insertHtml(f"<font color={css.COLOR['black']}><b>识别拼音:</b> {speech_pinyin}</font><br>")
        elif self.language == "Japanese":
            self.text_all_log.textCursor().insertHtml(f"<font color={css.COLOR['black']}><b>正确假名:</b> {true_kata}</font><br>")
            self.text_all_log.textCursor().insertHtml(f"<font color={css.COLOR['black']}><b>识别假名:</b> {speech_kata}</font><br>")
            self.text_all_log.textCursor().insertHtml(f"<font color={css.COLOR['black']}><b>正确罗马音:</b> {true_kata_romaji}</font><br>")
            self.text_all_log.textCursor().insertHtml(f"<font color={css.COLOR['black']}><b>识别罗马音:</b> {speech_kata_romaji}</font><br>")

        if wer < 0.2:
            self.text_all_log.textCursor().insertHtml(f"<font color={css.COLOR['green']}><b>匹配度:</b> {(1 - wer) * 100:.2f}%</font><br><br>")
        else:
            self.error_file_paths.append(file_path)
            self.text_error_log.textCursor().insertHtml(f"<font color={css.COLOR['black']}><b>错误文件:</b> {file_name}</font><br>")
            self.text_error_log.textCursor().insertHtml(f"<font color={css.COLOR['black']}><b>正确文本:</b> {true_text}</font><br>")
            self.text_error_log.textCursor().insertHtml(f"<font color={css.COLOR['black']}><b>识别文本:</b> {speech_text}</font><br>")
            if self.language == "Chinese":
                self.text_error_log.textCursor().insertHtml(f"<font color={css.COLOR['black']}><b>正确拼音:</b> {true_pinyin}</font><br>")
                self.text_error_log.textCursor().insertHtml(f"<font color={css.COLOR['black']}><b>识别拼音:</b> {speech_pinyin}</font><br>")
            elif self.language == "Japanese":
                self.text_error_log.textCursor().insertHtml(f"<font color={css.COLOR['black']}><b>正确假名:</b> {true_kata}</font><br>")
                self.text_error_log.textCursor().insertHtml(f"<font color={css.COLOR['black']}><b>识别假名:</b> {speech_kata}</font><br>")
                self.text_error_log.textCursor().insertHtml(f"<font color={css.COLOR['black']}><b>正确罗马音:</b> {true_kata_romaji}</font><br>")
                self.text_error_log.textCursor().insertHtml(f"<font color={css.COLOR['black']}><b>识别罗马音:</b> {speech_kata_romaji}</font><br>")
            if wer >= 0.5:
                self.text_all_log.textCursor().insertHtml(f"<font color={css.COLOR['red']}><b>匹配度:</b> {(1 - wer) * 100:.2f}%</font><br><br>")
                self.text_error_log.textCursor().insertHtml(f"<font color={css.COLOR['red']}><b>匹配度:</b> {(1 - wer) * 100:.2f}%</font><br><br>")

            elif 0.2 <= wer < 0.5:
                self.text_all_log.textCursor().insertHtml(f"<font color={css.COLOR['orange']}><b>匹配度:</b> {(1 - wer) * 100:.2f}</font><br><br>")
                self.text_error_log.textCursor().insertHtml(f"<font color={css.COLOR['orange']}><b>匹配度:</b> {(1 - wer) * 100:.2f}</font><br><br>")

        # 保持文本窗口的焦点跟随
        cursor_all_log = self.text_all_log.textCursor()
        cursor_all_log.movePosition(QTextCursor.End)
        self.text_all_log.setTextCursor(cursor_all_log)
        self.text_all_log.ensureCursorVisible()
        cursor_error_log = self.text_error_log.textCursor()
        cursor_error_log.movePosition(QTextCursor.End)
        self.text_error_log.setTextCursor(cursor_error_log)
        self.text_error_log.ensureCursorVisible()

        if total_index == total_file_num:
            self.error_log = self.text_error_log.toPlainText()
            if self.error_log:
                self.output_button.setEnabled(True)
            return

    def output_log_and_files(self):
        folder_name = os.path.basename(self.folder_path)
        output_text_speech_verify_result(folder_name, self.error_file_paths, self.error_log, self.language)
        self.output_button.setEnabled(False)
        self.notification_window = NotificationWindow('语音文本校验完毕', '输出成功, 错误文件和日志已保存至桌面')
        self.notification_window.show()
